/***************************************************************************************
*
* DFT.ASM
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This is a modified version of the SHARC dft.asm file that is included with
* the VisualDSP++ 3.5 software for 32-bit processors.
*
* This modified version shows a one-to-one conversion of the DFT SHARC code to 
* TigerSHARC. 
*
***************************************************************************************/

//#include "def21160.h"           /* Memory Mapped IOP register definitions */

#ifdef __ADSPTS201__
	#include <defts201.h>
#endif

#ifdef __ADSPTS101__
	#include <defts101.h>
#endif

#define N 64                    /* Constant for number of points in input */

//.SECTION/DM     dm_data;        /* Declare variables in data memory */
.SECTION     data1;      
.VAR input[N]= "test64.dat";
.VAR real[N];
.VAR imag[N];


//.SECTION/PM     pm_data;        /* Declare variables in program memory */
.SECTION     data2;
.VAR sine[N]= "sin64.dat";      /* Cosine is derived using a shifted */ 
                                /* pointer to this circular buffer.*/

//.SECTION/PM     pm_rsti;           /* The reset vector resides in this space */
//		NOP;
//		USTAT2= 0x108421;  /* 1st instr. to be executed after reset */
//		DM(WAIT)=USTAT2;   /* Set external memory waitstates to 0 */
//		JUMP start;
	     


//.SECTION/PM     pm_code;        /* Example setup for DFT routine */
.SECTION   program;
start:  
		j13=1;;		//M1=1;
		k13=1;;`	//M9=1;
		
		JB0=input;;	//B0=input;
		J0=JB0;;
		JL0=N;;//L0=@input;              /* Input buffer is circular */
		
		j5=imag;;	//I1=imag;
					//L1=0;
		//MODE1 = 0x1000000;		/* Enable Circular buffer */
		
		//CALL dft (DB);          /* Example delayed call instruction */
		j6=real;;	//I2=real;              /* In delay field of call */
		  			//L2=0;                 /*           ''            */
		call dft(P);;
end:            jump end;;

/*___________________________DFT Subroutine___________________________*/
dft:    
		KB0=sine;;		//B8=sine;                /* Sine pointer */
		K0=KB0;;
		KL0=N;;		//L8=@sine;
		
		KB1=sine;;		//B9=sine;                /* Derive cosine from sine by */
		K1=sine+N/4;;	//I9=sine+N/4;            /* shifting pointer over 2pi/4 */
		KL1=N;;		//L9=@sine;               /* and using a circular buffer.*/
		
		K2=0;;			//I10=0;                  /* I10 is used to increment the */
						//L10=0;                  /* frequency of sine lookup.*/
		xr15=0;;		//F15=0;                  /* Zero to clear accumulators */
		
		LC0=N;;			//LCNTR=N, DO outer UNTIL LCE;
outer:	xfr8=PASS r15; K12=K2;; 
		//F8=PASS F15, M8=I10;          /* Update frequency */
		xfr9=PASS r15; xr0=CB[j0+=j13]; xr5=CB[K1+=K12];; 
		//F9=PASS F15, F0=DM(J0,j13), F5=PM(I9,M8);
		xfr12=r0*r5; xr4=CB[k0+=k12];LC1=N-1;;
		//F12=F0*F5, F4=PM(K0,K12);
				 
		//LCNTR= N-1 , DO inner UNTIL LCE;
inner:	xfr13=r0*r4; xfr9=r9+r12; xr0=CB[j0+=j13]; xr5=CB[k1+=k12];;
		//F13=F0*F4, F9=F9+F12, F0=DM(J0,j13), F5=PM(K1,K12);
		 
		//inner: F12=F0*F5, F8=F8-F13, F4=PM(K0,K12);
		if NLC1E, jump inner(P);xfr12=r0*r5; xfr8=r8-r13; xr4=CB[k0+=k12];; 
		
 		xfr13=r0*r4; xfr9=r9+r12;;
		//F13=F0*F4, F9=F9+F12;
		xfr8=r8 - r13; [j6+=j13]= xr9;; 
		//F8=F8-F13, DM(J6,j13)=F9;      /* Write real result */
		K2=k2+k13;;
		//MODIFY(K2,k13);               /* Increment frequency */
		
		
		//outer: DM(J5,j13)=F8;         /* Write imaginary result */
		if NLC0E, jump outer;[j5+=j13]=xr8;;
		
		cjmp(ABS);;
		//RTS;


