/***************************************************************************************
*
* asm_sprt_TS.h
* Accompaniment for:
* Guide for Porting Code from the SHARC DSP Family to the TigerSHARC Processor Family
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* 26th May 2004 rev 1.0 --- AC
*
* This is a modified version of the asm_sprt.h file that is included with
* the VisualDSP++ 3.5 software for 32-bit processors. The original file
* contains macros from maintaining the stack and function entry and exit
* procedures.
*
* This modified version uses the same macro names to allow easier
* conversion from SHARC to TigerSHARC. The macros comply with the
* TigerSHARC C run-time environment
***************************************************************************************/

#ifndef __ASM_SPRT_DEFINED
#define __ASM_SPRT_DEFINED

#if !defined(__ECC__)  /* Don't parse for C code */

/******************************************************************************
* Function prologue for a function that calls another function
* This prologue stores the return address in CJMP to the stack. The prologue also 
* saves all compiler reserved registers. This is only necessary if these compiler 
* reserved registers are used in the assembly function, however they have been 
* saved by default to aid in the conversion process from SHARC to TigerSHARC code.
******************************************************************************/
#define entry	j26 = j27 - 0X40; k26 = k27 - 0x40;;\
				[j27+= 0xFFFFFFF0] = cjmp; k27 = k27 -0xF;; \
				q[j27 + 12] = j23:20; q[k27 +12] = k23:20;; \
				q[j27 + 8] = j19:16; q[k27 +8] = k19:16;; \
				q[j27 + 4] = XR31:28; q[k27 +4] = YR31:28;; \
				q[j27 += -4] = XR27:24; q[k27 += -4] = YR27:24;
				
/******************************************************************************
* Function prologue for a function that does not call another function
* This prologue does not store the return address in CJMP to the stack. The 
* prologue saves all compiler reserved registers. This is only necessary if these
* compiler reserved registers are used in the assembly function, however they have
* been saved by default to aid in the conversion process from SHARC to TigerSHARC
* code.
******************************************************************************/				
#define leaf_entry	j26 = j27 - 0X40; k26 = k27 - 0x40;;\
					j27 = j27 - 0x10; k27 = k27 - 0x10;; \
					q[j27 + 12] = j23:20; q[k27 +12] = k23:20;; \
					q[j27 + 8] = j19:16; q[k27 +8] = k19:16;; \
					q[j27 + 4] = XR31:28; q[k27 +4] = YR31:28;; \
					q[j27 += -4] = XR27:24; q[k27 += -4] = YR27:24;

/******************************************************************************
* Function epilogue for a function that does not call another function
* This epilogue does not restore the return address from the stack to CJMP..
* The epilogue restores all compiler reserved registers. This is only necessary 
* if these compiler reserved registers are used in the assembly function, however
* they have been saved by default to aid in the conversion process from SHARC to 
* TigerSHARC code.
******************************************************************************/						
#define leaf_exit 	XR27:24 = q[j27 + 4]; YR27:24 = q[k27 + 4];;\
					XR31:28 = q[j27 + 8]; YR31:28 = q[k27 + 8];;\
					j19:16 = q[j27 + 12]; k19:16 = q[k27 + 12];;\
					j23:20 = q[j27 + 16]; k23:20 = q[k27 + 16];;\
					cjmp(ABS)(np); j27:24 = q[j26+68]; k27:24 = q[k26+68];  

/******************************************************************************
* Function epilogue for a function that calls another function
* This epilogue restores the return address from the stack to CJMP. The epilogue
* also restores all compiler reserved registers. This is only necessary if these
* compiler reserved registers are used in the assembly function, however they 
* have been saved by default to aid in the conversion process from SHARC to 
* TigerSHARC code.
******************************************************************************/
#define exit		XR27:24 = q[j27 + 4]; YR27:24 = q[k27 + 4];;\
					XR31:28 = q[j27 + 8]; YR31:28 = q[k27 + 8];;\
					j19:16 = q[j27 + 12]; k19:16 = q[k27 + 12];;\
					j23:20 = q[j27 + 16]; k23:20 = q[k27 + 16];;\
					cjmp = [j26+64];;\
					cjmp(ABS)(np); j27:24 = q[j26+68]; k27:24 = q[k26+68];  


/******************************************************************************
* Used to place data onto the stack
******************************************************************************/
/* Used to place data onto the J stack */
#define puts	[j27+=-4]	/* push a single word onto the stack */
#define lputs	l[j27+=-4]	/* push a long word onto the stack */
#define qputs	q[j27+=-4]	/* push a quad word onto the stack */

/* Used to place data onto the J stack */
#define puts_jstack		puts	/* push a single word onto the stack */
#define lputs_jstack	lputs	/* push a long word onto the stack */
#define qputs_jstack	qputs	/* push a quad word onto the stack */

/* Used to place data onto the K stack */
#define puts_kstack		[k27+=-4]	/* push a single word onto the stack */
#define lputs_kstack	l[k27+=-4]	/* push a long word onto the stack */
#define qputs_kstack	q[k27+=-4]	/* push a quad word onto the stack */


/******************************************************************************
* Used to get data from the stack 												
* These macros do not modify the stack pointer they simply read the data from		
* the correct offset on the stack. For example gets(1) would read back the data 	
* that was written on the stack with the last "puts" macro 					
* It is important that the user uses the "alter(x)" macro after the data has 		
* been restored from the stack.
******************************************************************************/
/* Used to get data from the J stack */
#define gets(x)		[j27+(4*x)];	// read a single word from the stack
#define lgets(x)	l[j27+(4*x)];	// read a long word from the stack
#define qgets(x)	q[j27+(4*x)];	// read a quad word from the stack

/* Used to get data from the J stack */
#define gets_jstack(x)		gets(x)		// read a single word from the stack
#define lgets_jstack(x)		lgets(x)	// read a long word from the stack
#define qgets_jstack(x)		qgets(x)	// read a quad word from the stack

/* Used to get data from the K stack */
#define gets_kstack(x)		[k27+(4*x)];	// read a single word from the stack
#define lgets_kstack(x)		l[k27+(4*x)];	// read a long word from the stack
#define qgets_kstack(x)		q[k27+(4*x)];	// read a quad word from the stack


/******************************************************************************
* Modifies the stack pointer
* A positive value modifies the pointer down the stack (towards higher memory
* addresses). A negative value will result in more protected area being added
* to the stack allowing for data placement onto the stack using an offset load
* without modifying the stack pointer (Example j[27+4] = xstat;;
* 
* If this technique is used it is extrmely important to load the first value
* to j26+4. This leaves a quad word free at the top of the stack for the
* normal loads with post modify such as (puts). This is also important in the 
* case that an interrupt occurs as the handlers perform the context save using
* post modify loads.
******************************************************************************/
#define alter(x)	j27= j27+(4*x);
#define alter_jstack(x)	alter(x)
#define alter_kstack(x) k27 = k27+(4*x);


/******************************************************************************
* Retrieve passed parameters from the stack
* This macro retieves from the previous functions stack frame the parameters that
* are passed to the callee function. An offset of (75+x) is used for normal word
* reads as the first parameter passed on the stack is located at address 76.
* For long reads an offset of 74 is used and for quads an osset of 72, ensuring
* the required alignment. 
******************************************************************************/
#define reads(x)	[j26+ (75+x)];		// reads a normal parameter
#define lreads(x)	l[j26+ (74+(2*x))];	// reads a long word parameter
#define qreads(x)	q[j26+ (72+(4*x))];	// reads a quad word parameter


/******************************************************************************
* Performs the required action for calling another function from an assembly
* function.
******************************************************************************/
#define ccall(x)	IF true, CALL x; q[j27+0X4]=j27:24; q[k27+0X4]=k27:24;


/******************************************************************************
* Saves all the compute block registers to the stack 
******************************************************************************/
#define save_reg	qputs_jstack = xr3:0; qputs_kstack = yr3:0;;\
					qputs_jstack = xr7:4; qputs_kstack = yr7:4;;\
					qputs_jstack = xr11:8; qputs_kstack = yr11:8;;\
					qputs_jstack = xr15:12; qputs_kstack = yr15:12;;\
					qputs_jstack = xr19:16; qputs_kstack = yr19:16;;\
					qputs_jstack = xr23:20; qputs_kstack = yr23:20;;\
					qputs_jstack = xr27:24; qputs_kstack = yr27:24;;\
					qputs_jstack = xr31:28; qputs_kstack = yr31:28;

					
/******************************************************************************					
* Restores all the compute block registers from the stack 
******************************************************************************/										
#define restore_reg		xr31:28 = qgets_jstack(1) yr31:28 = qgets_kstack(1);\
						xr27:24 = qgets_jstack(2) yr27:24 = qgets_kstack(2);\
						xr23:20 = qgets_jstack(3) yr23:20 = qgets_kstack(3);\
						xr19:16 = qgets_jstack(4) yr19:16 = qgets_kstack(4);\
						xr15:12 = qgets_jstack(5) yr15:12 = qgets_kstack(5);\
						xr11:8 = qgets_jstack(6) yr11:8 = qgets_kstack(6);\
						xr7:4 = qgets_jstack(7) yr7:4 = qgets_kstack(7);\
						xr3:0 = qgets_jstack(8) yr3:0 = qgets_kstack(8);\
						alter_jstack(8) alter_kstack(8)


#endif  /* __ECC__ */ 

#endif  /* __ASM_SPRT_DEFINED */

