/***************************************************************************************
*
* DMA_Burst.ASM
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This is a TigerSHARC equivalent version of the SHARC DMA_Burst.asm file that is included with
* the VisualDSP++ 3.5 software for 32-bit processors.
*
***************************************************************************************/

#include <defts201.h>

#define N 16		// number of 32-bit words to transfer

.GLOBAL		_main;

.section sdram;
.var rx_buffer[N]; 

.section data1;
.var tx_buffer[N] =			0x00000000, 0x11111111, 0x22222222, 0x33333333, 
							0x44444444, 0x55555555, 0x66666666, 0x77777777, 
							0x88888888, 0x99999999, 0xaaaaaaaa, 0xbbbbbbbb, 
							0xcccccccc, 0xdddddddd, 0xeeeeeeee, 0xffffffff;

.section program;
start:
_main:
		// Initialize registers

		xr1 = 	SYSCON_MP_WID64 | SYSCON_MEM_WID64;;
		SYSCON = xr1;;	
		
		xr2 =	SDRCON_INIT | SDRCON_RAS2PC5 | SDRCON_PC2RAS2 | 
				SDRCON_REF3700 | SDRCON_PG256 | SDRCON_CLAT2 | SDRCON_ENBL;;
		SDRCON = xr2;;	


		// enable external port 0 DMA and global interupts

		j0 = j31 + _dma_int;;			// Set up DMA0 vector
		IVDMA0 = j0;;
	
	 	xR3 = IMASKL;;					// Read value of IMASKL
		xR4 = 0x00004000;;				// DMA0 interrupt
		xR5 = R3 or R4;;				// OR with contents of IMASKL
		IMASKL = xR5;;					// Write back to IMASKL
		
		xr0 = SQCTL;;
		xr0 = bset r0 by SQCTL_GIE_P;;					// enable global ints
		SQCTL = xr0;;

	/* Set up and perform DMA transfers */

										// External port transmit - DMA0
	xr0 = tx_buffer;;					// xr0 = source index = int. mem. DSP_0
	xr1 = 0x00100001;;					// count = 0x10, modify = 1 (normal word)
	xr3 = 0x43000000;;					// int mem,prio=norm,2D=no,word=32-bit,int=yes,RQ=enbl,chain=no
	DCS0 = xr3:0;;						// Set up source for transfer

	yr0 = rx_buffer;;					// xr0 = destination index = ext. mem. = SDRAM
	yr1 = 0x00100001;;					// count = 0x10, modify = 1 (normal word)
	yr3 = 0x83000000;;					// ext mem,prio=norm,2D=no,word=32-bits,int=yes,RQ=enbl,chain=no
	DCD0 = yr3:0;;						// Set up destination and start external port DMA0 transfer

	idle;;

	nop;;
	
wait0: jump wait0;;

_dma_int:  nop;;nop;;nop;; rti;;		


_main.end:

