/***************************************************************************************
*
* link port loopback.asm
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This is a TigerSHARC equivalent version of the SHARC Link Port Loopback example
* provided with this EE-note (see EE241.zip)
*
***************************************************************************************/

//******************************************************************************
// Macro Definitions & Includes
//******************************************************************************
#define N			8				//  Number of data words to transfer

#include "cache_macros.h"
#include <defTS201.h>
//******************************************************************************


//******************************************************************************
// Data internal memory block 2
//******************************************************************************
.section data2;

.align 4;
.var link_data_tx[N] = "tx_data.dat";	// storage for data contained within the file to be transmitted
.var link_data_rx[N];					// storage for data received through the link port

//******************************************************************************
// Program Code for internal memory block 0
//******************************************************************************
//***********************************************************************
.section reset;

// prepare cache
/*in the case of TS201, at the beginning of the program the 
cache must be enabled. The procedure is contained in the 
cache_enable macro that uses the refresh rate as input parameter
      -if CCLK=500MHz, refresh_rate=750
      -if CCLK=400MHz, refresh_rate=600
      -if CCLK=300MHz, refresh_rate=450
      -if CCLK=250MHz, refresh_rate=375
*/
	cache_enable(750);

	jump _start (NP);;	

//***********************************************************************
.section program;
_start:

//******************************************************************************
// Workaround for link port drive strength problem for rev 0.0a
//******************************************************************************
	J0 = J31 + 0xFF01;;
	TEST_MODES = J0;;
//******************************************************************************

/************************* Disable all link ports, DMA's and Vector Interrupts************************/
	XR0 = 0x00000000;;		
	XR1 = 0x00000000;;		
	XR2 = 0x00000000;;		

	LRCTL0 = XR0;;			// Link disable & clear
	LRCTL1 = XR0;;			// Link disable & clear
	LRCTL2 = XR0;;			// Link disable & clear
	LRCTL3 = XR0;;			// Link disable & clear
	LTCTL0 = XR0;;			// Link disable & clear
	LTCTL1 = XR0;;			// Link disable & clear
	LTCTL2 = XR0;;			// Link disable & clear
	LTCTL3 = XR0;;			// Link disable & clear

	XR3 = 0x00000000;;		// disable DMA
	DC4 = XR3:0;;			// disable link 0 tx DMA
	DC5 = XR3:0;;			// disable link 1 tx DMA
	DC6 = XR3:0;;			// disable link 2 tx DMA
	DC7 = XR3:0;;			// disable link 3 tx DMA
	DC8 = XR3:0;;			// disable link 0 rx DMA
	DC9 = XR3:0;;			// disable link 1 rx DMA
	DC10 = XR3:0;;			// disable link 2 rx DMA
	DC11 = XR3:0;;			// disable link 3 rx DMA

/*******************************************************************************************************/
	j0 = j31 + _dma_int;;	// Set DMA interrupt vector
	IVDMA11  = j0;;			// DMA11 interrupt---link 3 rx DMA

	xr1 = IMASKH;;						// enable link DMA interrupts
	xr1 = bset r1 by INT_DMA11_P;;
	IMASKH = xr1;;						// enable DMA11 interrupt---link 3 rx DMA

_Enable_HW_Interrupts:
	SQCTLST = SQCTL_GIE;;
_Enable_HW_SW_Interrupts.END:	

//******************************************************************************
// Setup LPORT:  LP3 TX,RX enabled.
//******************************************************************************
	xr0 = LRCTL_REN;; 	// enable link 3 (receive), 1 bit mode
	LRCTL3 = xr0;;					

	xr0 = LTCTL_TEN | LTCTL_TCLKDIV4;; 	// enable link 3 (transmit)
	LTCTL3 = xr0;;						// Tx enable, 1-bit mode, clk div by 4
//******************************************************************************

	
//******************************************************************************
// Setup DMA:  Transfer the contents of source to LBUF3TX -> LBUF3RX
//******************************************************************************
	xR0 = 0x00000000;;
	xR1 = 0x00000000;;
	xR2 = 0x00000000;;	
	xR3 = 0x00000000;;	
	
	xR1 = 0x00080004;;				// Count = N (8), Modifier = 4 (Quad)
	xR0 = link_data_rx;;
	xR2 = 0x00000000;;
	xr3 = TCB_INTMEM | TCB_QUAD | TCB_INT;;	// control word set for quad-word transfers to internal memory with interrupt enabled

	DC11 = xR3:0;;					// program TCB: Link port 3 receive TCB

	xR1 = 0x00080004;;				// Count = N (8), Modifier = 4 (Quad)
	xR0 = link_data_tx;;
	xR2 = 0x00000000;;
	xR3 = TCB_INTMEM | TCB_QUAD;; 	// Internal mem quad-word, int disabled
	
	DC7 = xR3:0;;					// program TCB: Link port 3 transmit TCB
	
//******************************************************************************
// wait for interrupt - DMA complete
//******************************************************************************
_wait:
	idle;;						// when DMA is completed an interrupt is generated which brings out of idle and jumps to _dma_int

wait0: jump wait0;;	
	
.align_code 4;
_dma_int:						// When tx and rx DMA's are completed an interrupt is generated. This is the isr
	nop;nop;nop;nop;;
	nop;nop;nop;nop;;
	nop;nop;nop;nop;;
	nop;nop;nop;nop;;
	rti(ABS)(NP);nop;nop;nop;;
_dma_int.END:	


